@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use vars qw($VERSION);
 
-$VERSION = '1.1';
+$VERSION = '1.3';
 
 require LWP::UserAgent;
 use Config;
@@ -26,7 +26,12 @@ my %randomness = (
 	    warn "Net::Random: Error talking to fourmilab.ch\n";
             return ();
 	}
-	map { map { hex } /(..)/g } grep { /^[0-9A-F]+$/ } split(/\s+/, $response->content());
+        my $content = $response->content();
+        if($content =~ /Error Generating HotBits/) {
+            warn("Net::Random: fourmilab.ch ran out of randomness for us\n");
+            return ();
+        }
+	map { map { hex } /(..)/g } grep { /^[0-9A-F]+$/ } split(/\s+/, $content);
     } },
     'random.org'   => { pool => [], retrieve => sub {
         my $response = $ua->get('http://www.random.org/cgi-bin/checkbuf');
@@ -36,9 +41,8 @@ my %randomness = (
 	} else {
 	    $response->content() =~ /^(\d+)/;
 	    if($1 < 20) {
-	        warn "Net::Random: random.org buffer nearly empty, pausing\n";
-	        sleep 15;
-	        # return &{$randomness{'random.org'}->{retrieve}};
+	        warn "Net::Random: random.org buffer nearly empty\n";
+	        return ();
             }
 	}
         $response = $ua->get(
@@ -96,8 +100,7 @@ honoured.
 While we always fetch 1024 bytes, data can be used up one, two, three or
 four bytes at a time, depending on the range between the minimum and
 maximum desired values.  There may be a noticeable delay while more
-random data is fetched.  Warnings may be emitted in case of network
-problems.
+random data is fetched.
 
 The maintainers of both randomness sources claim that their data is
 *truly* random.  A some simple tests show that they are certainly more
@@ -152,7 +155,10 @@ specified, defaults to 1.
 
 If it fails to retrieve data, we return undef.  Note that fourmilab.ch
 rations random data and you are only permitted to retrieve a certain
-amount of randomness in any 24 hour period.
+amount of randomness in any 24 hour period, and random.org asks software
+authors to not empty their randomness pool entirely.  In both these cases
+we spit out a warning.  See the section on ERROR
+HANDLING below.
 
 =cut
 
@@ -217,6 +223,27 @@ is sufficiently random and unique for my own uses, but I can not recommend
 that you do too.  As in any security situation, you need to perform your own
 risk analysis.
 
+=head1 ERROR HANDLING
+
+There are two types of error that this module can emit which aren't your
+fault.  Those are network
+errors, in which case it emits a warning:
+
+    Net::Random: Error talking to [your source]
+
+and errors generated by the randomness sources, which look like:
+
+    Net::Random: [your source] [message]
+
+Once you hit either of these errors, it means that either you have run
+out of randomness and can't get any more, or you are very close to
+running out of randomness.  Because this module's raison d'&ecirc;tre
+is to provide a source of truly random data when you don't have your
+own one available, it does not provide any pseudo-random fallback.
+
+If you want to implement your own fallback, you can catch those warnings
+by using C<$SIG{__WARN__}>.  See C<perldoc perlvar> for details.
+
 =head1 FEEDBACK
 
 I welcome feedback about my code, especially constructive criticism.
@@ -237,7 +264,7 @@ Suggestions from the following people have been included:
 
 =head1 COPYRIGHT
 
-Copyright 2003 - 2004 David Cantrell
+Copyright 2003 - 2007 David Cantrell
 
 This module is free-as-in-speech software, and may be used, distributed,
 and modified under the same terms as Perl itself.
